<?php
namespace KnitPay\Gateways\Cashfree;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Pronamic\WordPress\Pay\Core\PaymentMethod;
use Pronamic\WordPress\Pay\Payments\Payment;
use Pronamic\WordPress\Pay\Payments\PaymentStatus;
use Exception;

/**
 * Title: Cashfree Gateway
 * Copyright: 2020-2023 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since 2.4
 */
class Gateway extends Core_Gateway {

	const NAME = 'cashfree';

	/**
	 * Initializes an Cashfree gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function init( Config $config ) {
		$this->set_method( self::METHOD_HTTP_REDIRECT );

		// Supported features.
		$this->supports = [
			'payment_status_request',
		];

		$this->test_mode = 0;
		if ( self::MODE_TEST === $config->mode ) {
			$this->test_mode = 1;
		}

		$this->config = $config;
		$this->api    = new API( $config->api_id, $config->secret_key, $this->test_mode );

		$this->register_payment_methods();
	}

	private function register_payment_methods() {
		$this->register_payment_method( new PaymentMethod( PaymentMethods::CREDIT_CARD ) );
		$this->register_payment_method( new PaymentMethod( PaymentMethods::CASHFREE ) );
	}

	/**
	 * Get available payment methods.
	 *
	 * @return array<int, string>
	 * @see Core_Gateway::get_available_payment_methods()
	 */
	public function get_available_payment_methods() {
		return $this->get_supported_payment_methods();
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		if ( ! defined( 'KNIT_PAY_CASHFREE' ) ) {
			$error = sprintf(
				/* translators: 1: Cashfree */
				__( 'Knit Pay supports %1$s with a Premium Addon. But you can get this premium addon for free and also you can get a special discount on transaction fees. Visit the Knit Pay website (knitpay.org) to know more.', 'knit-pay' ),
				__( 'Cashfree', 'knit-pay' )
			);
			throw new Exception( $error );
		}

		$this->cashfree_order_id = $payment->key . '_' . $payment->get_id();
		$order_link              = $this->api->create_order_link( $this->get_payment_data( $payment ) );

		$payment->set_transaction_id( $this->cashfree_order_id );
		$payment->set_action_url( $order_link );
	}

	/**
	 * Get Payment Data.
	 *
	 * @param Payment $payment
	 *            Payment.
	 *
	 * @return array
	 */
	private function get_payment_data( Payment $payment ) {

		$customer        = $payment->get_customer();
		$billing_address = $payment->get_billing_address();
		$customer_phone  = $this->config->default_customer_phone;
		if ( ! empty( $billing_address ) && ! empty( $billing_address->get_phone() ) ) {
			$customer_phone = $this->format_phone_number( $billing_address->get_phone() );
		}

		$order_id       = $this->cashfree_order_id;
		$order_amount   = $payment->get_total_amount()->number_format( null, '.', '' );
		$order_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		$order_note     = $payment->get_description();
		$customer_email = $customer->get_email();
		$return_url     = add_query_arg(
			[
				'order_id'    => '{order_id}',
				'order_token' => '{order_token}',
			],
			$payment->get_return_url()
		);
		$notify_url     = add_query_arg( 'kp_cashfree_webhook', '', home_url( '/' ) );
		$cust_id        = 'CUST_' . $payment->get_order_id() . '_' . $payment->get_id();


		// @see https://docs.cashfree.com/reference/createorder
		return [
			'order_id'         => $order_id,
			'order_amount'     => $order_amount,
			'order_currency'   => $order_currency,
			'customer_details' => [
				'customer_id'    => $cust_id,
				'customer_email' => $customer_email,
				'customer_phone' => $customer_phone,
			],
			'order_meta'       => [
				'return_url'      => $return_url,
				'notify_url'      => $notify_url,
				'payment_methods' => '', // TODO pass specific payment gateway
			],
			'order_note'       => $order_note,
		];
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
		if ( PaymentStatus::SUCCESS === $payment->get_status() || PaymentStatus::EXPIRED === $payment->get_status() ) {
			return;
		}

		$order_status = $this->api->get_order_status( $payment->get_transaction_id() );

		if ( isset( $_POST['txStatus'] ) && Statuses::CANCELLED === $_POST['txStatus'] ) {
			$payment->set_status( PaymentStatus::CANCELLED );
			return;
		}

		if ( isset( $order_status->txStatus ) ) {
			$order_txn_status = $order_status->txStatus;

			if ( Statuses::SUCCESS === $order_txn_status ) {
				$payment->set_transaction_id( $order_status->referenceId );
			}

			$payment->set_status( Statuses::transform( $order_txn_status ) );
			$payment->add_note( 'Cashfree Transaction Status: ' . $order_txn_status . '<br>Order Status: ' . $order_status->orderStatus . '<br>Transaction reference ID: ' . $order_status->referenceId );
		}

		if ( isset( $order_status->orderExpiryTime ) ) {
			$expiry_date = new \DateTime( $order_status->orderExpiryTime, new \DateTimeZone( 'Asia/Kolkata' ) );
			$payment->set_expiry_date( $expiry_date );
			if ( time() > $expiry_date->getTimestamp() ) {
				$payment->set_status( PaymentStatus::EXPIRED );
			}
		}
	}

	private function format_phone_number( $customer_phone ) {
		// Remove - or whitespace.
		$customer_phone = preg_replace( '/[\s\-]+/', '', $customer_phone );

		// Remove 0 from beginning of phone number.
		$customer_phone = 10 < strlen( $customer_phone ) ? ltrim( $customer_phone, '0' ) : $customer_phone;

		return $customer_phone;
	}
}
